import unittest

from mongomock.store import RWLock


class LockTestCase(unittest.TestCase):

    def test_rwlock_exception(self):
        """Asserts exceptions occur between a lock's acquire/release"""
        lock = RWLock()

        for method in [lock.reader, lock.writer]:
            try:
                with method():
                    raise ValueError
            except ValueError:
                pass

            # Accessing private attributes but oh well
            self.assertFalse(lock._no_writers.locked())
            self.assertFalse(lock._no_readers.locked())



import os
import time
import unittest
from unittest import TestCase, skipIf, skipUnless

import mongomock
import mongomock.gridfs
from mongomock import helpers
from packaging import version

try:
    import gridfs
    from gridfs import errors
    _HAVE_GRIDFS = True
except ImportError:
    _HAVE_GRIDFS = False


try:
    from bson.objectid import ObjectId

    import pymongo
    from pymongo import MongoClient as PymongoClient
except ImportError:
    ...


@skipUnless(helpers.HAVE_PYMONGO, 'pymongo not installed')
@skipUnless(_HAVE_GRIDFS and hasattr(gridfs.__builtins__, 'copy'), 'gridfs not installed')
@skipIf(os.getenv('NO_LOCAL_MONGO'), 'No local Mongo server running')
class GridFsTest(TestCase):

    @classmethod
    def setUpClass(cls):
        mongomock.gridfs.enable_gridfs_integration()

    def setUp(self):
        super(GridFsTest, self).setUp()
        self.fake_conn = mongomock.MongoClient()
        self.mongo_conn = self._connect_to_local_mongodb()
        self.db_name = 'mongomock___testing_db'

        self.mongo_conn[self.db_name]['fs']['files'].drop()
        self.mongo_conn[self.db_name]['fs']['chunks'].drop()

        self.real_gridfs = gridfs.GridFS(self.mongo_conn[self.db_name])
        self.fake_gridfs = gridfs.GridFS(self.fake_conn[self.db_name])

    def tearDown(self):
        super(GridFsTest, self).setUp()
        self.mongo_conn.close()
        self.fake_conn.close()

    def test__put_get_small(self):
        before = time.time()
        fid = self.fake_gridfs.put(GenFile(50))
        rid = self.real_gridfs.put(GenFile(50))
        after = time.time()
        ffile = self.fake_gridfs.get(fid)
        rfile = self.real_gridfs.get(rid)
        self.assertEqual(ffile.read(), rfile.read())
        fake_doc = self.get_fake_file(fid)
        mongo_doc = self.get_mongo_file(rid)
        self.assertSameFile(mongo_doc, fake_doc, max_delta_seconds=after - before + 1)

    def test__put_get_big(self):
        # 500k files are bigger than doc size limit
        before = time.time()
        fid = self.fake_gridfs.put(GenFile(500000, 10))
        rid = self.real_gridfs.put(GenFile(500000, 10))
        after = time.time()
        ffile = self.fake_gridfs.get(fid)
        rfile = self.real_gridfs.get(rid)
        self.assertEqual(ffile.read(), rfile.read())
        fake_doc = self.get_fake_file(fid)
        mongo_doc = self.get_mongo_file(rid)
        self.assertSameFile(mongo_doc, fake_doc, max_delta_seconds=after - before + 1)

    def test__delete_exists_small(self):
        fid = self.fake_gridfs.put(GenFile(50))
        self.assertTrue(self.get_fake_file(fid) is not None)
        self.assertTrue(self.fake_gridfs.exists(fid))
        self.fake_gridfs.delete(fid)
        self.assertFalse(self.fake_gridfs.exists(fid))
        self.assertFalse(self.get_fake_file(fid) is not None)
        # All the chunks got removed
        self.assertEqual(0, self.fake_conn[self.db_name].fs.chunks.count_documents({}))

    def test__delete_exists_big(self):
        fid = self.fake_gridfs.put(GenFile(500000))
        self.assertTrue(self.get_fake_file(fid) is not None)
        self.assertTrue(self.fake_gridfs.exists(fid))
        self.fake_gridfs.delete(fid)
        self.assertFalse(self.fake_gridfs.exists(fid))
        self.assertFalse(self.get_fake_file(fid) is not None)
        # All the chunks got removed
        self.assertEqual(0, self.fake_conn[self.db_name].fs.chunks.count_documents({}))

    def test__delete_no_file(self):
        # Just making sure we don't crash
        self.fake_gridfs.delete(ObjectId())

    def test__list_files(self):
        fids = [self.fake_gridfs.put(GenFile(50, 9), filename='one'),
                self.fake_gridfs.put(GenFile(62, 5), filename='two'),
                self.fake_gridfs.put(GenFile(654, 1), filename='three'),
                self.fake_gridfs.put(GenFile(5), filename='four')]
        names = ['one', 'two', 'three', 'four']
        names_no_two = [x for x in names if x != 'two']
        for x in self.fake_gridfs.list():
            self.assertIn(x, names)

        self.fake_gridfs.delete(fids[1])

        for x in self.fake_gridfs.list():
            self.assertIn(x, names_no_two)

        three_file = self.get_fake_file(fids[2])
        self.assertEqual('three', three_file['filename'])
        self.assertEqual(654, three_file['length'])
        self.fake_gridfs.delete(fids[0])
        self.fake_gridfs.delete(fids[2])
        self.fake_gridfs.delete(fids[3])
        self.assertEqual(0, len(self.fake_gridfs.list()))

    def test__find_files(self):
        fids = [self.fake_gridfs.put(GenFile(50, 9), filename='a'),
                self.fake_gridfs.put(GenFile(62, 5), filename='b'),
                self.fake_gridfs.put(GenFile(654, 1), filename='b'),
                self.fake_gridfs.put(GenFile(5), filename='a')]
        c = self.fake_gridfs.find({'filename': 'a'}).sort('uploadDate', -1)
        should_be_fid3 = c.next()
        should_be_fid0 = c.next()
        self.assertFalse(c.alive)

        self.assertEqual(fids[3], should_be_fid3._id)
        self.assertEqual(fids[0], should_be_fid0._id)

    def test__put_exists(self):
        self.fake_gridfs.put(GenFile(1), _id='12345')
        with self.assertRaises(errors.FileExists):
            self.fake_gridfs.put(GenFile(2, 3), _id='12345')

    def assertSameFile(self, real, fake, max_delta_seconds=1):
        # https://pymongo.readthedocs.io/en/stable/migrate-to-pymongo4.html#disable-md5-parameter-is-removed
        if helpers.PYMONGO_VERSION < version.parse('4.0'):
            self.assertEqual(real['md5'], fake['md5'])

        self.assertEqual(real['length'], fake['length'])
        self.assertEqual(real['chunkSize'], fake['chunkSize'])
        self.assertLessEqual(
            abs(real['uploadDate'] - fake['uploadDate']).seconds, max_delta_seconds,
            msg='real: %s, fake: %s' % (real['uploadDate'], fake['uploadDate']))

    def get_mongo_file(self, i):
        return self.mongo_conn[self.db_name]['fs']['files'].find_one({'_id': i})

    def get_fake_file(self, i):
        return self.fake_conn[self.db_name]['fs']['files'].find_one({'_id': i})

    def _connect_to_local_mongodb(self, num_retries=60):
        """Performs retries on connection refused errors (for travis-ci builds)"""
        for retry in range(num_retries):
            if retry > 0:
                time.sleep(0.5)
            try:
                return PymongoClient(
                    host=os.environ.get('TEST_MONGO_HOST', 'localhost'), maxPoolSize=1
                )
            except pymongo.errors.ConnectionFailure as e:
                if retry == num_retries - 1:
                    raise
                if 'connection refused' not in e.message.lower():
                    raise


class GenFile(object):
    def __init__(self, length, value=0, do_encode=True):
        self.gen = self._gen_data(length, value)
        self.do_encode = do_encode

    def _gen_data(self, length, value):
        while length:
            length -= 1
            yield value

    def _maybe_encode(self, s):
        if self.do_encode and isinstance(s, str):
            return s.encode('UTF-8')
        return s

    def read(self, num_bytes=-1):
        s = ''
        if num_bytes <= 0:
            bytes_left = -1
        else:
            bytes_left = num_bytes
        while True:
            n = next(self.gen, None)
            if n is None:
                return self._maybe_encode(s)
            s += chr(n)
            bytes_left -= 1
            if bytes_left == 0:
                return self._maybe_encode(s)


if __name__ == '__main__':
    unittest.main()



import unittest

import mongomock


class NotImplementedTests(unittest.TestCase):

    def tearDown(self):
        mongomock.warn_on_feature('session')

    def test_raises(self):
        collection = mongomock.MongoClient().db.collection
        with self.assertRaises(NotImplementedError):
            collection.insert_one({}, session=True)

    def test_ignores(self):
        mongomock.ignore_feature('session')

        collection = mongomock.MongoClient().db.collection
        collection.insert_one({}, session=True)

    def test_on_and_off(self):
        collection = mongomock.MongoClient().db.collection

        with self.assertRaises(NotImplementedError):
            collection.insert_one({'_id': 1}, session=True)

        mongomock.ignore_feature('session')

        collection.insert_one({'_id': 2}, session=True)

        mongomock.warn_on_feature('session')

        with self.assertRaises(NotImplementedError):
            collection.insert_one({'_id': 3}, session=True)

        self.assertEqual({2}, {doc['_id'] for doc in collection.find()})

    def test_wrong_key(self):
        with self.assertRaises(KeyError):
            mongomock.ignore_feature('sessions')



from .diff import diff
from collections import OrderedDict
import copy
import functools
from mongomock.helpers import RE_TYPE


_COMPARE_EXCEPTIONS = 'exceptions'


class MultiCollection(object):

    def __init__(self, conns):
        super(MultiCollection, self).__init__()
        self.conns = conns.copy()
        self.do = Foreach(self.conns, compare=False)
        self.compare = Foreach(self.conns, compare=True)
        self.compare_ignore_order = Foreach(
            self.conns, compare=True, ignore_order=True)
        self.compare_exceptions = Foreach(self.conns, compare=_COMPARE_EXCEPTIONS)


class Foreach(object):

    def __init__(self, objs, compare, ignore_order=False,
                 method_result_decorators=()):
        self.___objs = objs
        self.___compare = compare
        self.___ignore_order = ignore_order
        self.___decorators = list(method_result_decorators)
        self.___sort_by = None

    def __getattr__(self, method_name):
        return ForeachMethod(
            self.___objs,
            self.___compare,
            self.___ignore_order,
            method_name,
            self.___decorators,
            self.___sort_by)

    def sort_by(self, fun):
        self.___sort_by = fun
        return self

    def __call__(self, *decorators):
        return Foreach(
            self.___objs,
            self.___compare,
            self.___ignore_order,
            self.___decorators + list(decorators))


class ForeachMethod(object):

    def __init__(self, objs, compare, ignore_order, method_name, decorators, sort_by):
        self.___objs = objs
        self.___compare = compare
        self.___ignore_order = ignore_order
        self.___method_name = method_name
        self.___decorators = decorators
        self.___sort_by = sort_by

    def _call(self, obj, args, kwargs):
        # copying the args and kwargs is important, because pymongo changes
        # the dicts (fits them with the _id)
        return self.___apply_decorators(
            getattr(obj, self.___method_name)(*_deepcopy(args), **_deepcopy(kwargs)))

    def _get_exception_type(self, obj, args, kwargs, name):
        try:
            self._call(obj, args, kwargs)
            assert False, 'No exception raised for ' + name
        except Exception as err:
            return type(err)

    def __call__(self, *args, **kwargs):
        if self.___compare == _COMPARE_EXCEPTIONS:
            results = dict(
                (name, self._get_exception_type(obj, args, kwargs, name=name))
                for name, obj in self.___objs.items()
            )
        else:
            results = dict(
                (name, self._call(obj, args, kwargs))
                for name, obj in self.___objs.items()
            )
        if self.___compare:
            _assert_no_diff(results, ignore_order=self.___ignore_order, sort_by=self.___sort_by)
        return results

    def ___apply_decorators(self, obj):
        for d in self.___decorators:
            obj = d(obj)
        return obj


def _assert_no_diff(results, ignore_order, sort_by):
    if _result_is_cursor(results) or _result_is_command_cursor(results):
        # If we were given a sort function, use that.
        if sort_by is not None:
            value_processor = functools.partial(_expand_cursor, sort=ignore_order, by=sort_by)
        else:
            value_processor = functools.partial(_expand_cursor, sort=ignore_order)
    else:
        assert not ignore_order
        value_processor = None
    prev_name = prev_value = None
    for index, (name, value) in enumerate(results.items()):
        if value_processor is not None:
            value = value_processor(value)
        if index > 0:
            d = diff(prev_value, value)
            assert not d, _format_diff_message(prev_name, name, d)
        prev_name = name
        prev_value = value


def _result_is_cursor(results):
    return any(type(result).__name__ == 'Cursor' for result in results.values())


def _result_is_command_cursor(results):
    return any(type(result).__name__ == 'CommandCursor'
               for result in results.values())


def by_id(document):
    return str(document.get('_id', str(document)))


def _expand_cursor(cursor, sort, by=by_id):
    returned = [result.copy() for result in cursor]
    if sort:
        returned.sort(key=by)
    for result in returned:
        result.pop('_id', None)
    return returned


def _format_diff_message(a_name, b_name, diff_list):
    msg = 'Unexpected Diff:'
    for (path, a_value, b_value) in diff_list:
        a_path = [a_name] + path
        b_path = [b_name] + path
        msg += '\n\t{} != {} ({} != {})'.format(
            '.'.join(map(str, a_path)), '.'.join(
                map(str, b_path)), a_value, b_value)
    return msg


def _deepcopy(x):
    """Deepcopy, but ignore regex objects..."""
    if isinstance(x, RE_TYPE):
        return x
    if isinstance(x, list) or isinstance(x, tuple):
        return type(x)(_deepcopy(y) for y in x)
    if isinstance(x, (dict, OrderedDict)):
        return type(x)((_deepcopy(k), _deepcopy(v)) for k, v in x.items())
    return copy.deepcopy(x)



from tests.diff import diff
from unittest import TestCase


class DiffTest(TestCase):
    def test__assert_no_diff(self):
        for obj in [
                1,
                'string',
                {'complex': {'object': {'with': ['lists']}}},
        ]:
            self.assertEqual(diff(obj, obj), [])

    def test__diff_values(self):
        self._assert_entire_diff(1, 2)
        self._assert_entire_diff('a', 'b')

    def test__diff_sequences(self):
        self._assert_entire_diff([], [1, 2, 3])

    def test__composite_diff(self):
        a = {'a': {'b': [1, 2, 3]}}
        b = {'a': {'b': [1, 6, 3]}}
        [(path, x, y)] = diff(a, b)
        self.assertEqual(path, ['a', 'b', 1])
        self.assertEqual(x, 2)
        self.assertEqual(y, 6)

    def _assert_entire_diff(self, a, b):
        [(_, x, y)] = diff(a, b)
        self.assertEqual(x, a)
        self.assertEqual(y, b)



from packaging import version
import sys
import unittest
from unittest import skipIf, skipUnless

try:
    from unittest import mock
    _HAVE_MOCK = True
except ImportError:
    try:
        import mock
        _HAVE_MOCK = True
    except ImportError:
        _HAVE_MOCK = False

import mongomock
from mongomock import helpers

try:
    from bson import codec_options
    from pymongo.read_preferences import ReadPreference
except ImportError:
    pass


class MongoClientApiTest(unittest.TestCase):

    def test__read_preference(self):
        client = mongomock.MongoClient()
        self.assertEqual('Primary', client.read_preference.name)
        self.assertEqual(client.read_preference, client.db.read_preference)
        self.assertEqual(client.read_preference, client.db.coll.read_preference)

        client2 = mongomock.MongoClient(read_preference=client.read_preference)
        self.assertEqual(client2.read_preference, client.read_preference)

        with self.assertRaises(TypeError):
            mongomock.MongoClient(read_preference=0)

    @unittest.skipIf(not helpers.HAVE_PYMONGO, 'pymongo not installed')
    def test__different_read_preference(self):
        client = mongomock.MongoClient(read_preference=ReadPreference.NEAREST)
        self.assertEqual(ReadPreference.NEAREST, client.db.read_preference)
        self.assertEqual(ReadPreference.NEAREST, client.db.coll.read_preference)

    @unittest.skipIf(not helpers.HAVE_PYMONGO, 'pymongo not installed')
    def test__codec_options_with_pymongo(self):
        client = mongomock.MongoClient()
        self.assertEqual(codec_options.CodecOptions(), client.codec_options)
        self.assertFalse(client.codec_options.tz_aware)

    def test__codec_options(self):
        client = mongomock.MongoClient()
        self.assertFalse(client.codec_options.tz_aware)

        client = mongomock.MongoClient(tz_aware=True)
        self.assertTrue(client.codec_options.tz_aware)
        self.assertTrue(client.db.collection.codec_options.tz_aware)

        with self.assertRaises(TypeError):
            mongomock.MongoClient(tz_aware='True')

    def test__parse_url(self):
        client = mongomock.MongoClient('mongodb://localhost:27017/')
        self.assertEqual(('localhost', 27017), client.address)

        client = mongomock.MongoClient('mongodb://localhost:1234,example.com/')
        self.assertEqual(('localhost', 1234), client.address)

        client = mongomock.MongoClient('mongodb://example.com,localhost:1234/')
        self.assertEqual(('example.com', 27017), client.address)

        client = mongomock.MongoClient('mongodb://[::1]:1234/')
        self.assertEqual(('::1', 1234), client.address)

        with self.assertRaises(ValueError):
            mongomock.MongoClient('mongodb://localhost:1234:456/')

        with self.assertRaises(ValueError):
            mongomock.MongoClient('mongodb://localhost:123456/')

        with self.assertRaises(ValueError):
            mongomock.MongoClient('mongodb://localhost:mongoport/')

    def test__equality(self):
        self.assertEqual(
            mongomock.MongoClient('mongodb://localhost:27017/'),
            mongomock.MongoClient('mongodb://localhost:27017/'))
        self.assertEqual(
            mongomock.MongoClient('mongodb://localhost:27017/'),
            mongomock.MongoClient('localhost'))
        self.assertNotEqual(
            mongomock.MongoClient('/var/socket/mongo.sock'),
            mongomock.MongoClient('localhost'))

    @skipIf(sys.version_info < (3,), 'Older versions of Python do not handle hashing the same way')
    @skipUnless(
        helpers.PYMONGO_VERSION < version.parse('3.12'),
        "older versions of pymongo didn't have proper hashing")
    def test__not_hashable(self):
        with self.assertRaises(TypeError):
            {mongomock.MongoClient('localhost')}  # pylint: disable=expression-not-assigned

    @skipIf(sys.version_info < (3,), 'Older versions of Python do not handle hashing the same way')
    @skipIf(
        helpers.PYMONGO_VERSION < version.parse('3.12'),
        "older versions of pymongo didn't have proper hashing")
    def test__hashable(self):
        {mongomock.MongoClient('localhost')}  # pylint: disable=expression-not-assigned

    def test__parse_hosts(self):
        client = mongomock.MongoClient('localhost')
        self.assertEqual(('localhost', 27017), client.address)

        client = mongomock.MongoClient('localhost:1234,example.com')
        self.assertEqual(('localhost', 1234), client.address)

        client = mongomock.MongoClient('example.com,localhost:1234')
        self.assertEqual(('example.com', 27017), client.address)

        client = mongomock.MongoClient('[::1]:1234')
        self.assertEqual(('::1', 1234), client.address)

        client = mongomock.MongoClient('/var/socket/mongo.sock')
        self.assertEqual(('/var/socket/mongo.sock', None), client.address)

        with self.assertRaises(ValueError):
            mongomock.MongoClient('localhost:1234:456')

        with self.assertRaises(ValueError):
            mongomock.MongoClient('localhost:123456')

        with self.assertRaises(ValueError):
            mongomock.MongoClient('localhost:mongoport')

    @unittest.skipIf(not _HAVE_MOCK, 'mock not installed')
    def test_database_names(self):
        client = mongomock.MongoClient()
        client.one_db.my_collec.insert_one({})

        if helpers.PYMONGO_VERSION >= version.parse('4.0'):
            with self.assertRaises(TypeError):
                client.database_names()
            return

        with mock.patch('warnings.warn') as mock_warn:
            self.assertEqual(['one_db'], client.database_names())

        self.assertEqual(1, mock_warn.call_count)
        self.assertIn('deprecated', mock_warn.call_args[0][0])

    def test_list_database_names(self):
        client = mongomock.MongoClient()
        self.assertEqual([], client.list_database_names())

        # Query a non existant collection.
        client.one_db.my_collec.find_one()
        self.assertEqual([], client.list_database_names())

        client.one_db.my_collec.insert_one({})
        self.assertEqual(['one_db'], client.list_database_names())

    def test_client_implements_context_managers(self):
        with mongomock.MongoClient() as client:
            client.one_db.my_collec.insert_one({})
            result = client.one_db.my_collec.find_one({})
            self.assertTrue(result)

    def test_start_session(self):
        client = mongomock.MongoClient()
        with self.assertRaises(NotImplementedError):
            client.start_session()

    @mock.patch('mongomock.SERVER_VERSION', '3.6')
    def test_server_version(self):
        client = mongomock.MongoClient()
        server_info = client.server_info()
        self.assertEqual('3.6', server_info['version'])
        self.assertEqual([3, 6, 0, 0], server_info['versionArray'])

    def test_consistent_server_version(self):
        client = mongomock.MongoClient()
        server_info = client.server_info()
        with mock.patch('mongomock.SERVER_VERSION', '3.6'):
            self.assertEqual(server_info, client.server_info())



import collections
import datetime
from packaging import version
import sys
from unittest import TestCase, skipIf, skipUnless

import mongomock
from mongomock import helpers
from mongomock import read_concern

try:
    from bson import codec_options
    from pymongo.read_preferences import ReadPreference
except ImportError:
    pass


class UTCPlus2(datetime.tzinfo):
    def fromutc(self, dt):
        return dt + self.utcoffset(dt)

    def tzname(self, dt):
        return '<dummy UTC+2>'

    def utcoffset(self, dt):
        return datetime.timedelta(hours=2)

    def dst(self, dt):
        return datetime.timedelta()


class DatabaseAPITest(TestCase):

    def setUp(self):
        self.database = mongomock.MongoClient().somedb

    def test__get_collection_by_attribute_underscore(self):
        with self.assertRaises(AttributeError) as err_context:
            self.database._users  # pylint: disable=pointless-statement

        self.assertIn("Database has no attribute '_users'", str(err_context.exception))

        # No problem accessing it through __get_item__.
        self.database['_users'].insert_one({'a': 1})
        self.assertEqual(1, self.database['_users'].find_one().get('a'))

    def test__session(self):
        with self.assertRaises(NotImplementedError):
            self.database.list_collection_names(session=1)
        with self.assertRaises(NotImplementedError):
            self.database.drop_collection('a', session=1)
        with self.assertRaises(NotImplementedError):
            self.database.create_collection('a', session=1)
        with self.assertRaises(NotImplementedError):
            self.database.dereference(_DBRef('somedb', 'a', 'b'), session=1)

    def test__command_ping(self):
        self.assertEqual({'ok': 1}, self.database.command({'ping': 1}))

    def test__command_ping_string(self):
        self.assertEqual({'ok': 1}, self.database.command('ping'))

    def test__command_fake_ping_string(self):
        with self.assertRaises(NotImplementedError):
            self.assertEqual({'ok': 1}, self.database.command('a_nice_ping'))

    def test__command(self):
        with self.assertRaises(NotImplementedError):
            self.database.command({'count': 'user'})

    def test__repr(self):
        self.assertEqual(
            "Database(mongomock.MongoClient('localhost', 27017), 'somedb')", repr(self.database))

    def test__rename_unknown_collection(self):
        with self.assertRaises(mongomock.OperationFailure):
            self.database.rename_collection('a', 'b')

    def test__dereference(self):
        self.database.a.insert_one({'_id': 'b', 'val': 42})
        doc = self.database.dereference(_DBRef('somedb', 'a', 'b'))
        self.assertEqual({'_id': 'b', 'val': 42}, doc)

        self.assertEqual(None, self.database.dereference(_DBRef('somedb', 'a', 'a')))
        self.assertEqual(None, self.database.dereference(_DBRef('somedb', 'b', 'b')))

        with self.assertRaises(ValueError):
            self.database.dereference(_DBRef('otherdb', 'a', 'b'))

        with self.assertRaises(TypeError):
            self.database.dereference('b')

    def test__read_preference(self):
        self.assertEqual('Primary', self.database.read_preference.name)
        self.assertEqual(self.database.collection.read_preference, self.database.read_preference)

        with self.assertRaises(TypeError):
            self.database.get_collection('a', read_preference='nearest')

    @skipIf(not helpers.HAVE_PYMONGO, 'pymongo not installed')
    def test__get_collection_different_read_preference(self):
        database = mongomock.MongoClient()\
            .get_database('somedb', read_preference=ReadPreference.NEAREST)
        self.assertEqual('Nearest', database.read_preference.name)
        self.assertEqual(database.read_preference, database.collection.read_preference)

        col = database.get_collection('col', read_preference=ReadPreference.PRIMARY)
        self.assertEqual('Primary', col.read_preference.name)

    @skipIf(not helpers.HAVE_PYMONGO, 'pymongo not installed')
    def test__get_collection_different_codec_options(self):
        database = mongomock.MongoClient().somedb
        a = database.get_collection('a', codec_options=codec_options.CodecOptions(tz_aware=True))
        self.assertTrue(a.codec_options.tz_aware)

    @skipIf(not helpers.HAVE_PYMONGO, 'pymongo not installed')
    def test__codec_options(self):
        self.assertEqual(codec_options.CodecOptions(), self.database.codec_options)

    @skipIf(not helpers.HAVE_PYMONGO, 'pymongo not installed')
    def test__read_concern(self):
        self.assertEqual(read_concern.ReadConcern(), self.database.read_concern)

    def test__with_options(self):
        with self.assertRaises(NotImplementedError):
            self.database.with_options(write_concern=3)

    @skipIf(not helpers.HAVE_PYMONGO, 'pymongo not installed')
    def test__with_options_pymongo(self):
        other = self.database.with_options(read_preference=self.database.NEAREST)
        self.assertFalse(other is self.database)

        self.database.coll.insert_one({'_id': 42})
        self.assertEqual({'_id': 42}, other.coll.find_one())

        self.database.with_options(codec_options=codec_options.CodecOptions())
        self.database.with_options()

        self.database.with_options(codec_options=codec_options.CodecOptions(tz_aware=True))

        tz_aware_db = mongomock.MongoClient(tz_aware=True).somedb
        self.assertIs(
            tz_aware_db,
            tz_aware_db.with_options(codec_options=codec_options.CodecOptions(tz_aware=True)))

        custom_document_class = codec_options.CodecOptions(document_class=collections.OrderedDict)
        with self.assertRaises(NotImplementedError):
            self.database.with_options(custom_document_class)

        custom_uuid_representation = codec_options.CodecOptions(uuid_representation=4)
        with self.assertRaises(NotImplementedError):
            self.database.with_options(custom_uuid_representation)

        custom_unicode_error_hander = codec_options.CodecOptions(
            unicode_decode_error_handler='ignore')
        with self.assertRaises(NotImplementedError):
            self.database.with_options(custom_unicode_error_hander)

        custom_tzinfo = codec_options.CodecOptions(tz_aware=True, tzinfo=UTCPlus2())
        with self.assertRaises(NotImplementedError):
            self.database.with_options(custom_tzinfo)

    @skipIf(
        not helpers.HAVE_PYMONGO or helpers.PYMONGO_VERSION < version.parse('3.8'),
        'pymongo not installed or <3.8')
    def test__with_options_type_registry(self):
        class _CustomTypeCodec(codec_options.TypeCodec):
            @property
            def python_type(self):  # pylint: disable=invalid-overridden-method
                return _CustomTypeCodec

            def transform_python(self, unused_value):
                pass

            @property
            def bson_type(self):  # pylint: disable=invalid-overridden-method
                return int

            def transform_bson(self, unused_value):
                pass

        custom_type_registry = codec_options.CodecOptions(
            type_registry=codec_options.TypeRegistry([_CustomTypeCodec()]))
        with self.assertRaises(NotImplementedError):
            self.database.with_options(custom_type_registry)

    def test__collection_names(self):
        self.database.create_collection('a')
        self.database.create_collection('b')

        if helpers.PYMONGO_VERSION >= version.parse('4.0'):
            with self.assertRaises(TypeError):
                self.database.collection_names()
            return

        self.assertEqual(set(self.database.collection_names()), set(['a', 'b']))

        self.database.c.drop()
        self.assertEqual(set(self.database.collection_names()), set(['a', 'b']))

    def test__list_collection_names(self):
        self.database.create_collection('a')
        self.database.create_collection('b')
        self.assertEqual(set(self.database.list_collection_names()), set(['a', 'b']))

        self.database.c.drop()
        self.assertEqual(set(self.database.list_collection_names()), set(['a', 'b']))

    def test__list_collections(self):
        self.database.create_collection('a')

        with self.assertRaises(NotImplementedError):
            self.database.list_collections()

    def test__create_collection(self):
        coll = self.database.create_collection('c')
        self.assertIs(self.database.c, coll)
        self.assertRaises(mongomock.CollectionInvalid,
                          self.database.create_collection, 'c')

    def test__create_collection_bad_names(self):
        with self.assertRaises(TypeError):
            self.database.create_collection(3)
        with self.assertRaises(TypeError):
            self.database[3]  # pylint: disable=pointless-statement

        bad_names = (
            '',
            'foo..bar',
            '...',
            '$foo',
            '.foo',
            'bar.',
            'foo\x00bar',
        )
        for name in bad_names:
            with self.assertRaises(mongomock.InvalidName, msg=name):
                self.database.create_collection(name)
            with self.assertRaises(mongomock.InvalidName, msg=name):
                self.database[name]  # pylint: disable=pointless-statement

    def test__lazy_create_collection(self):
        col = self.database.a
        self.assertEqual(set(self.database.list_collection_names()), set())
        col.insert_one({'foo': 'bar'})
        self.assertEqual(set(self.database.list_collection_names()), set(['a']))

    def test__equality(self):
        self.assertEqual(self.database, self.database)
        client = mongomock.MongoClient('localhost')
        self.assertNotEqual(client.a, client.b)
        self.assertEqual(client.a, client.get_database('a'))
        self.assertEqual(client.a, mongomock.MongoClient('localhost').a)
        self.assertNotEqual(client.a, mongomock.MongoClient('example.com').a)

    @skipIf(sys.version_info < (3,), 'Older versions of Python do not handle hashing the same way')
    @skipUnless(
        helpers.PYMONGO_VERSION < version.parse('3.12'),
        "older versions of pymongo didn't have proper hashing")
    def test__not_hashable(self):
        with self.assertRaises(TypeError):
            {self.database}  # pylint: disable=pointless-statement

    @skipIf(sys.version_info < (3,), 'Older versions of Python do not handle hashing the same way')
    @skipIf(
        helpers.PYMONGO_VERSION < version.parse('3.12'),
        "older versions of pymongo didn't have proper hashing")
    def test__hashable(self):
        {self.database}  # pylint: disable=pointless-statement

    def test__bad_type_as_a_read_concern_returns_type_error(self):
        client = mongomock.MongoClient()
        with self.assertRaises(
            TypeError, msg='read_concern must be an instance of pymongo.read_concern.ReadConcern'
        ):
            mongomock.database.Database(client, 'foo', None, read_concern='bar')


_DBRef = collections.namedtuple('DBRef', ['database', 'collection', 'id'])



import time
import unittest

import mongomock

try:
    import pymongo
    _HAVE_PYMONGO = True
except ImportError:
    _HAVE_PYMONGO = False

try:
    from unittest import mock
except ImportError:
    import mock

import platform
_USING_PYPY = platform.python_implementation() == 'PyPy'


@unittest.skipIf(not _HAVE_PYMONGO, 'pymongo not installed')
@unittest.skipIf(_USING_PYPY, 'PyPy does not handle mocking time sleep properly')
class PatchTest(unittest.TestCase):
    """Test the use of the patch function.

    Test functions in this test are embedded in inner function so that the
    patch decorator are only called at testing time.
    """

    @mongomock.patch()
    def test__decorator(self):
        client1 = pymongo.MongoClient()
        client1.db.coll.insert_one({'name': 'Pascal'})

        client2 = pymongo.MongoClient()
        self.assertEqual(['db'], client2.list_database_names())
        self.assertEqual('Pascal', client2.db.coll.find_one()['name'])
        client2.db.coll.drop()

        self.assertEqual(None, client1.db.coll.find_one())

    @mongomock.patch(on_new='create')
    def test__create_new(self):
        client1 = pymongo.MongoClient('myserver.example.com', port=12345)
        client1.db.coll.insert_one({'name': 'Pascal'})

        client2 = pymongo.MongoClient(host='myserver.example.com', port=12345)
        self.assertEqual('Pascal', client2.db.coll.find_one()['name'])

    @mongomock.patch()
    def test__error_new(self):
        # Valid because using the default server which was whitelisted by default.
        pymongo.MongoClient()

        with self.assertRaises(ValueError):
            pymongo.MongoClient('myserver.example.com', port=12345)

    @mongomock.patch((
        'mongodb://myserver.example.com:12345',
        'mongodb://otherserver.example.com:27017/default-db',
        'mongodb://[2001:67c:2e8:22::c100:68b]',
        'mongodb://[2001:67c:2e8:22::c100:68b]:1234',
        'mongodb://r1.example.net:27017,r2.example.net:27017/',
        '/var/lib/mongo.sock',
    ))
    def test__create_servers(self):
        pymongo.MongoClient('myserver.example.com', port=12345)
        pymongo.MongoClient('otherserver.example.com')
        pymongo.MongoClient('[2001:67c:2e8:22::c100:68b]')
        pymongo.MongoClient('mongodb://[2001:67c:2e8:22::c100:68b]:27017/base')
        pymongo.MongoClient('[2001:67c:2e8:22::c100:68b]', port=1234)
        pymongo.MongoClient('r1.example.net')
        pymongo.MongoClient('/var/lib/mongo.sock')

        with self.assertRaises(ValueError):
            pymongo.MongoClient()

    @mongomock.patch(on_new='timeout')
    @mock.patch(time.__name__ + '.sleep')
    def test__create_timeout(self, mock_sleep):
        pymongo.MongoClient()

        mock_sleep.reset_mock()

        with self.assertRaises(pymongo.errors.ServerSelectionTimeoutError):
            client = pymongo.MongoClient('myserver.example.com', port=12345)
            client.db.coll.insert_one({'name': 'Pascal'})

        mock_sleep.assert_called_once_with(30000)

    @mongomock.patch('example.com')
    def test__different_default_db(self):
        client_1 = pymongo.MongoClient('mongodb://example.com/db1')
        client_2 = pymongo.MongoClient('mongodb://example.com/db2')

        # Access the same data from different clients, despite the different DB.
        client_1.test_db.collection.insert_one({'name': 'Pascal'})
        self.assertEqual(['Pascal'], [d['name'] for d in client_2.test_db.collection.find()])

        # Access the data from "default DB" of client 1 but by its name from client 2.
        client_1.get_default_database().collection.insert_one({'name': 'Lascap'})
        self.assertEqual(['Lascap'], [d['name'] for d in client_2.db1.collection.find()])

        # Access the data from "default DB" of client 2 but by its name from client 1.
        client_2.get_default_database().collection.insert_one({'name': 'Caribou'})
        self.assertEqual(['Caribou'], [d['name'] for d in client_1.db2.collection.find()])

    @mongomock.patch(('my-db_client-url',))
    def test__rename_through_another_client(self):
        client1 = pymongo.MongoClient('mongodb://my-db_client-url/test')
        client1.test.my_collec.insert_one({'_id': 'Previous data'})

        client2 = pymongo.MongoClient('mongodb://my-db_client-url/test')
        client2.test.drop_collection('my_collec')
        client2.test.other_collec.insert_one({'_id': 'New data'})
        client2.test.other_collec.rename('my_collec')

        self.assertEqual(['New data'], [d['_id'] for d in client1.test.my_collec.find()])

    @mongomock.patch(servers=(('server.example.com', 27017),))
    def test__tuple_server_host_and_port(self):
        objects = [dict(votes=1), dict(votes=2)]
        client = pymongo.MongoClient('server.example.com')
        client.db.collection.insert_many(objects)

        collection = pymongo.MongoClient('server.example.com').db.collection
        for document in collection.find():
            collection.update_one(document, {'$set': {'votes': document['votes'] + 1}})

        self.assertEqual([2, 3], sorted(d.get('votes') for d in client.db.collection.find()))


if __name__ == '__main__':
    unittest.main()
